/**
 ********************************************************************
 * @file    psdk_typedef.h
 * @version V2.0.0
 * @date    2019/08/30
 * @brief   PSDK type define file.
 *
 * @copyright (c) 2018-2019 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef PSDK_TYPEDEF_H
#define PSDK_TYPEDEF_H

/* Includes ------------------------------------------------------------------*/
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Exported constants --------------------------------------------------------*/

/* Exported types ------------------------------------------------------------*/
/**
 * @brief Type define double as psdk_f64_t.
 */
typedef double psdk_f64_t;
/**
 * @brief Type define float as psdk_f32_t.
 */
typedef float psdk_f32_t;
/**
 * @brief Type define uint64 as T_PsdkReturnCode.
 */
typedef uint64_t T_PsdkReturnCode;

#define PSDK_RETURN_CODE_OK                 0 /*!< Payload SDK return code represents as status is ok. */
#define PSDK_RETURN_CODE_ERR_ALLOC          1 /*!< Payload SDK return code represents as status alloc error. */
#define PSDK_RETURN_CODE_ERR_TIMEOUT        2 /*!< Payload SDK return code represents as status timeout error. */
#define PSDK_RETURN_CODE_ERR_NOT_FOUND      3 /*!< Payload SDK return code represents as status not found error. */
#define PSDK_RETURN_CODE_ERR_OUT_OF_RANGE   4 /*!< Payload SDK return code represents as status out of range error. */
#define PSDK_RETURN_CODE_ERR_PARAM          5 /*!< Payload SDK return code represents as status parameter error. */
#define PSDK_RETURN_CODE_ERR_SYSTEM         6 /*!< Payload SDK return code represents as status system error. */
#define PSDK_RETURN_CODE_ERR_BUSY           7 /*!< Payload SDK return code represents as status busy error. */
#define PSDK_RETURN_CODE_ERR_UNSUPPORT      8 /*!< Payload SDK return code represents as status unsupport error. */
#define PSDK_RETURN_CODE_ERR_UNKNOWN        0xFF /*!< Payload SDK return code represents as status unknown error. */

#pragma pack (1)

typedef struct Vector3d {
    int32_t x; /*!< Specifies int32 value of x for vector. */
    int32_t y; /*!< Specifies int32 value of y for vector. */
    int32_t z; /*!< Specifies int32 value of z for vector. */
} T_PsdkVector3d;

typedef struct Vector3f {
    psdk_f32_t x; /*!< Specifies float value of x for vector. */
    psdk_f32_t y; /*!< Specifies float value of y for vector. */
    psdk_f32_t z; /*!< Specifies float value of z for vector. */
} T_PsdkVector3f;

typedef struct {
    int32_t pitch; /*!< Specifies int32 value of pitch for attitude. */
    int32_t roll; /*!< Specifies int32 value of roll for attitude */
    int32_t yaw; /*!< Specifies int32 value of yaw for attitude */
} T_PsdkAttitude3d;

#pragma pack ()

/* define compiler specific symbols */
#if defined (__ICCARM__)
#elif defined (__CC_ARM)
#pragma anon_unions
#elif defined (__GNUC__)
#elif defined (__TASKING__)
#endif

#ifdef __cplusplus
}
#endif

#endif //PSDK_TYPEDEF_H
/************************ (C) COPYRIGHT DJI Innovations *******END OF FILE******/
