/**
 ********************************************************************
 * @file    performance_monitor.h
 * @version V2.0.0
 * @date    2019/11/10
 * @brief   This is the header file for "performance_monitor.c", defining the structure and
 * (exported) function prototypes.
 *
 * @copyright (c) 2018-2019 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef PERFORMANCE_MONITOR_H
#define PERFORMANCE_MONITOR_H

/* Includes ------------------------------------------------------------------*/
#include "psdk_typedef.h"

#ifdef __cplusplus
extern "C" {
#endif

/* Exported constants --------------------------------------------------------*/


/* Exported types ------------------------------------------------------------*/
typedef struct {
    unsigned int user;
    unsigned int nice;
    unsigned int system;
    unsigned int idle;
} T_TotalCpuOccupy;

typedef struct {
    pid_t pid;
    unsigned int utime;
    unsigned int stime;
    unsigned int cutime;
    unsigned int cstime;
} T_ProcessCpuOccupy;

/* Exported functions --------------------------------------------------------*/
int getPhyMem(pid_t p);
int getTotalMem();
unsigned int getCpuTotalOccupy();
unsigned int getCpuOccupyOfProcess(pid_t pid);
unsigned int getCpuOccupyOfThread(pid_t pid, pid_t tid);
float getPcpuOfThread(pid_t pid, pid_t tid);
float getPmem(pid_t p);
unsigned int getThreadCountOfProcess(pid_t pid);
void getTidListOfProcess(pid_t pid, pid_t *tidList, unsigned int size);
void getNameOfThread(pid_t pid, pid_t tid, char *name, unsigned int size);
unsigned int getHeapUsed(pid_t pid);
unsigned int getStackUsed(pid_t pid);

#ifdef __cplusplus
}
#endif

#endif // PERFORMANCE_MONITOR_H

/************************ (C) COPYRIGHT DJI Innovations *******END OF FILE******/
