/**
 ********************************************************************
 * @file    psdk_hal.c
 * @version V2.0.0
 * @date    2019/07/01
 * @brief
 *
 * @copyright (c) 2018-2019 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "hal_uart.h"

/* Private constants ---------------------------------------------------------*/
#define LINUX_UART_DEV    "/dev/ttyUSB0"

/* Private types -------------------------------------------------------------*/
static int s_psdkUartFd = -1;

/* Private functions declaration ---------------------------------------------*/

/* Exported functions definition ---------------------------------------------*/
T_PsdkReturnCode Hal_UartInit(void)
{
    struct termios options;
    T_PsdkReturnCode returnCode = PSDK_RETURN_CODE_OK;

    s_psdkUartFd = open(LINUX_UART_DEV, O_RDWR | O_NOCTTY | O_NDELAY);
    if (s_psdkUartFd == -1) {
        return PSDK_RETURN_CODE_ERR_SYSTEM;
    }

    if (tcgetattr(s_psdkUartFd, &options) != 0) {
        close(s_psdkUartFd);
        return PSDK_RETURN_CODE_ERR_SYSTEM;
    }

    cfsetispeed(&options, B460800);
    cfsetospeed(&options, B460800);

    options.c_cflag |= CLOCAL;
    options.c_cflag |= CREAD;
    options.c_cflag &= ~CRTSCTS;
    options.c_cflag &= ~CSIZE;
    options.c_cflag |= CS8;
    options.c_cflag &= ~PARENB;
    options.c_iflag &= ~INPCK;
    options.c_cflag &= ~CSTOPB;
    options.c_oflag &= ~OPOST;
    options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
    options.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
    options.c_cc[VTIME] = 0;
    options.c_cc[VMIN] = 0;

    tcflush(s_psdkUartFd, TCIFLUSH);

    if (tcsetattr(s_psdkUartFd, TCSANOW, &options) != 0) {
        close(s_psdkUartFd);
        return PSDK_RETURN_CODE_ERR_SYSTEM;
    }

    return returnCode;
}

T_PsdkReturnCode Hal_UartSendData(const uint8_t *pBuf, uint16_t bufLen)
{
    int32_t realLen;

    if (s_psdkUartFd == -1) {
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }

    realLen = write(s_psdkUartFd, pBuf, bufLen);
    if (realLen == bufLen) {
        return PSDK_RETURN_CODE_OK;
    } else {
        return PSDK_RETURN_CODE_ERR_UNKNOWN;
    }
}

T_PsdkReturnCode Hal_UartReadData(uint8_t *pBuf, uint16_t bufLen, uint16_t *realBufLen)
{
    *realBufLen = read(s_psdkUartFd, pBuf, bufLen);

    return PSDK_RETURN_CODE_OK;
}

/* Private functions definition-----------------------------------------------*/

/****************** (C) COPYRIGHT DJI Innovations *****END OF FILE****/
